<%@ Page Language="C#" Debug="true" %>

<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="System.Net" %>
<%@ Import Namespace="System.Net.Security" %>
<%@ Import Namespace="System.Security.Cryptography.X509Certificates" %>

<script runat="server">
	private bool DummyCertificateValidator(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
	{
		return true;
	}

    protected void Page_Load(object sender, EventArgs e) {
        // Load settings
        bool ssl = false;
        if (ConfigurationManager.AppSettings["PINsafeSecure"] != null) {
            ssl = ConfigurationManager.AppSettings["PINsafeSecure"].ToLower().Equals("true");
        }

        string server = "localhost";
        if (ConfigurationManager.AppSettings["PINsafeServer"] != null) {
            server = ConfigurationManager.AppSettings["PINsafeServer"];
        }

        int port = 8080;
        if (ConfigurationManager.AppSettings["PINsafePort"] != null) {
            try {
                port = Int32.Parse(ConfigurationManager.AppSettings["PINsafePort"]);
            }
            catch (FormatException) {}
        }

        string context = "pinsafe";
        if (ConfigurationManager.AppSettings["PINsafeContext"] != null) {
            context = ConfigurationManager.AppSettings["PINsafeContext"];
        }

        // Construct URL for image
        StringBuilder url = ssl ? new StringBuilder("https://") : new StringBuilder("http://");

        string username = Request.QueryString["username"];
        if (String.IsNullOrEmpty(username)) {
            // No username: return empty image;
            return;
        }
        string sIgnoreDomain = ConfigurationManager.AppSettings["PINsafeIgnoreDomain"];
        bool ignoreDomain = true;
        if (sIgnoreDomain != null && sIgnoreDomain.ToLower().Equals("false")) {
            ignoreDomain = false;
        }
        if (ignoreDomain) {
            int loc = username.IndexOf('\\');
            if (loc >= 0) {
                username = username.Substring(loc + 1);
            }
            else {
                loc = username.IndexOf('@');
                if (loc > 0) {
                    username = username.Substring(0, loc);
                }
            }
        }

        string requestType = Request["type"];
        string imageServlet = (String.Compare(requestType, "message", true) == 0) ? "DCMessage" : "SCImage";
        url.AppendFormat("{0}:{1}/{2}/{3}?username={4}", server, port, context, imageServlet, username);

        // Request image
        bool acceptSelfSigned = false;
        if (ConfigurationManager.AppSettings["PINsafeAcceptSelfSigned"] != null) {
            acceptSelfSigned = ConfigurationManager.AppSettings["PINsafeAcceptSelfSigned"].ToLower().Equals("true");
        }
        if (acceptSelfSigned) {
            ServicePointManager.ServerCertificateValidationCallback = DummyCertificateValidator;
        }
        WebRequest imgRequest = WebRequest.Create(url.ToString());
        WebResponse imgResponse = imgRequest.GetResponse();
        if (imgResponse != null) {
            int imgSize = 0;
            Stream imgStream = imgResponse.GetResponseStream();
            if (imgStream != null) {
                try {

                    // Read image data from stream and write it to client
                    byte[] imgBuffer = new byte[1024];
                    int read = 0;
                    do {
                        imgSize += read;
                        read = imgStream.Read(imgBuffer, 0, imgBuffer.Length);
                        Response.OutputStream.Write(imgBuffer, 0, read);
                    } while (read > 0);
                } finally {
                    // Close image stream
                    imgStream.Close();
                }
            }

            // Set correct content type
            Response.ContentType = imgResponse.ContentType;

            // Set correct content length
            Response.AddHeader("Content-Length", imgSize.ToString());
        }

        // Try and prevent caching
        Response.CacheControl = "private";
        Response.ExpiresAbsolute = new DateTime(1970, 1, 1);
        Response.AddHeader("Pragma", "no-cache");

        // Done
        Response.End();
    }

</script>
